import json
from sawsi.aws import shared


class IAM:
    def __init__(self, boto3_session, region=shared.DEFAULT_REGION):
        self.client = boto3_session.client('iam')
        self.region = region


    def create_role(self, role_name, trust_policy, description='SAWSI Generated Role'):
        """
        IAM 역할 생성
        :param role_name:
        :param trust_policy:
        :param description
        :return:
        """
        role_response = self.client.create_role(
            RoleName=role_name,
            AssumeRolePolicyDocument=json.dumps(trust_policy),
            Description=description
        )
        return role_response


    def put_role_policy(self, role_name, policy_name, policy):
        """
        # 인라인 정책을 IAM 역할에 연결
        :param role_name:
        :param policy_name:
        :param policy:
        :return:
        """
        response = self.client.put_role_policy(
            RoleName=role_name,
            PolicyName=policy_name,
            PolicyDocument=json.dumps(policy)
        )
        return response

    def get_role(self, role_name):
        response = self.client.get_role(RoleName=role_name)
        return response



import json
from typing import Literal


class Exp:
    def __init__(self,
                 field=None, value=None, condition:Literal['eq', 'neq', 'lte', 'lt', 'gte', 'gt', 'btw', 'stw', 'is_in', 'contains', 'exist', 'not_exist']=None,
                 operation:Literal['or', 'and']=None, left=None, right=None):
        """
        DynamoDB Filter Expression (Recursive Version)
        :param field:
        :param value:
        :param condition: 'eq' | 'neq' | 'lte' | 'lt' | 'gte' | 'gt' | 'btw' | 'stw' |
                        'is_in' | 'contains' | 'exist' | 'not_exist'
        :param operation:
        :param left:
        :param right:
        """
        allow_conditions = {'eq', 'neq', 'lte', 'lt', 'gte', 'gt', 'btw', 'stw', 'is_in', 'contains', 'exist', 'not_exist'}
        allow_operations = {'or', 'and'}
        if condition not in allow_conditions:
            raise Exception(f"condition(={condition}) not in {allow_conditions}")
        if operation not in allow_operations:
            raise Exception(f'operation(={operation}) not in {allow_operations}')

        self.field = field
        self.value = value
        self.condition = condition
        self.operation = operation
        self.left = left
        self.right = right

    def to_dict(self):
        if self.operation:
            return {
                'left': self.left.to_dict(),
                'operation': self.operation,
                'right': self.right.to_dict(),
            }
        else:
            return {
                'field': self.field,
                'value': self.value,
                'condition': self.condition,
            }

    def __repr__(self):
        return json.dumps(self.to_dict())

    def or_(self, other):
        return Exp(operation='or', left=self, right=other)

    def and_(self, other):
        return Exp(operation='and', left=self, right=other)


if __name__ == '__main__':
    # Example usage
    expression = Exp('name1', 'example', 'contains').or_(
        Exp('name2', 'example2', 'contains').and_(
            Exp('name3', 'example3', 'contains')
        )
    )

    print(expression)



"""
This is sample code to tell how to use sawsi framework.
You could delete this file after know how to use this framework.
"""
from {{app}}.dao import sample_user_dao
from {{app}}.model.sample_user import User
from {{app}}.view_model import sample_user_vm
from pydantic import validate_call


@validate_call
def login(email:str, password:str):
    return {
        'session_id': '<SESSION_ID>'
    }


@validate_call  # session_id 에 int 등이 등어가면 에러가 레이즈됩니다.
def get_me(session_id:str):
    # Get Data Model from DAO
    user:User = sample_user_dao.get_user_by_session(session_id)
    # Transform to view model
    user_view_model:dict = sample_user_vm.make(user)
    return {
        'user': user_view_model
    }




class SMS:
    def __init__(self, boto3_session, region_name='us-east-1'):
        self.client = boto3_session.client('sns', region_name=region_name)
        self.region = boto3_session.region_name

    def send_sms(self, phone, message):
        response = self.client.response = self.client.publish(
            PhoneNumber=phone,  # 실제 전화번호로 변경
            Message=message,
        )
        return response
